package br.com.linx.sisdia.fiscal.sped;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.PrintStream;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 
 * Carrega o arquivo de unload gerado pelo sped1000, faz as conversões e
 * complementações necessárias e gera o arquivo
 * 
 * @author ricardo.xavier
 *
 */

public class DescarregaSped extends CmdSped {

	private Connection bd;
//	private PreparedStatement stmtCtdescri;
//	private PreparedStatement stmtModescri;
	private Map<Integer, String> mapaCt;
	private Map<Integer, String> mapaMo;

	// jdbc:informix-sqli://A-SRVBH012:30070/sisdia:informixserver=ol_dasjd01;Client_Locale=pt_br.8859-1;;user=tomcat;password=703kg7d14
	public static void main(String[] args) throws Exception {

		if (args.length != 4) {
			System.out.println("Uso: Balancete <url> <arqUnload> <codEmpresa> <arqSaida>");
			return;
		}		
		
		// argumentos
		String url = args[0];
		String arqUnload = args[1];
		int codEmpresa = Integer.parseInt(args[2]);
		String arqSaida = args[3];

		DescarregaSped sped = new DescarregaSped();
		sped.setComando("descarrega");

		sped.processa(arqUnload, arqSaida, url, codEmpresa);

	}

	private void processa(String arqUnload, String arqSaida, String url, int codEmpresa) throws Exception {

		try {

			Date t1 = new Date();
			bd = conectaBanco(url);
			carregaDiretorio(bd, codEmpresa);

			BufferedReader reader = new BufferedReader(new FileReader(getDiretorio() + arqUnload));

			PrintStream saida = new PrintStream(getDiretorio() + arqSaida, "ISO-8859-1");

			carregaHistoricos();

			gravaPasso("Processando " + getDiretorio() + arqUnload);

			int contador = 0;
			String linha;
			while ((linha = reader.readLine()) != null) {

				if ((contador % 10000) == 0) {
					gravaPasso("Registros processados: " + contador);
				}
				contador++;

				String[] campos = linha.split(";");
				Registro reg = new Registro();
				reg.setIdLc(Integer.parseInt(campos[4]));
				reg.setArLc(campos[5]);
				reg.setDados(campos[6]);

				String dados = null;
				if (reg.getIdLc() > 0) {
					String complemento = carregaComplemento(reg.getIdLc(), reg.getArLc());
					dados = reg.getDados() + complemento;
				} else {
					dados = reg.getDados();
				}

				saida.printf("%s\r\n", dados);
			}

			reader.close();
			saida.close();
			gravaPasso("Registros processados: " + contador);

			Date t2 = new Date();
			gravaPasso("tempo(min)=" + ((t2.getTime() - t1.getTime()) / 60000));
			gravaPasso("ok");

		} catch (Exception e) {
			e.printStackTrace();
			PrintStream result = new PrintStream(
					getDiretorio() + (isDebug() ? "descarrega_debug.out" : "descarrega.out"));
			result.println("erro: " + e.getMessage());
			result.close();
			throw e;

		} finally {
			if (bd != null) {
				try {
					bd.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}
		}

	}
	
	private void carregaHistoricos() throws FileNotFoundException, SQLException {	
		gravaPasso("Carregando historicos");		
		mapaCt = new HashMap<Integer, String>();
		mapaMo = new HashMap<Integer, String>();		
		carregaHistoricos("A");
		carregaHistoricos("?");
	}

	private void carregaHistoricos(String arLc) throws FileNotFoundException, SQLException {
		
		String sql = String.format("select descrhist, id_lancto, nrolinhadescr \n" 
				+ "from %s \n" 
				+ "order by id_lancto, nrolinhadescr",
				arLc.equals("A") ? "ctdescri" : "modescri");
		Statement stmt = bd.createStatement();
		StringBuilder sb = null;
		int ultId = -1;
		ResultSet cursor = stmt.executeQuery(sql);
		while (cursor.next()) {
			int id = cursor.getInt("id_lancto");
			String descrhist = cursor.getString("descrhist");
			if ((id != ultId)) {
				if (ultId != -1) {
					if (arLc.equals("A")) {
						mapaCt.put(ultId, sb.toString());
					} else {
						mapaMo.put(ultId, sb.toString());
					}
				}
				ultId = id;
				sb = new StringBuilder();
			}
			if ((descrhist != null) && (descrhist.trim().length() > 0)) {
				sb.append(" " + rtrim(descrhist));
			}
		}
		cursor.close();
		
	}
	
	private String rtrim(String s) {
		int pos = 0;
		for (int i=0; i<s.length(); i++) {
			if (s.charAt(i) == ' ') {
				pos++;
			} else {
				break;
			}
		}
		return pos == 0 ? s.trim() : s.substring(0, pos) + s.substring(pos).trim();
	}

	private String carregaComplemento(int idLc, String arLc) throws SQLException {

		String descrhist = arLc.equals("A") ? mapaCt.get(idLc) : mapaMo.get(idLc);
		if (descrhist != null) {
			return SpedUtils.substituiCaracteresInvalidos(descrhist) + "||";
		}		
		
		return "||";
	}

}
